'''
Created on 26 avr. 2018

@author: Tonneau Stephane
'''

import os
import types
from datetime import datetime
from pidclient.logging_factory import LoggingFactory
try:
    from airflow import DAG
except ImportError:
    # necessary to allow sphinx generate documentation with airflow is not available
    print("ERROR : Missing required airflow package")

    class DAG():
        """Class definition to avoid module initialisation exception"""
        def __init__(self):
            raise Exception("airflow import failed")

AIRFLOW_ENV_ID = "PIDCLIENT_PARENTID"


class AirflowOperatorIssue(Exception):
    """Exception definition to allow user to make distinction between retry and failure.
    This exception is used when retrying operator.
    """
    def __init__(self, message, is_retry, errors=None):
        # Call the base class constructor with the parameters it needs
        super(AirflowOperatorIssue, self).__init__(message)
        self.retry = is_retry

        # Now for your custom code...
        self.errors = errors

def pid_pre_execute(self, context):
    """method called before the execution of a task

    :param context: the context
    :type context: context
    """
    addendum_information = {}
    if u'task_instance_key_str' in context:
        tmp = str(context[u'task_instance_key_str'])
        tmp = tmp.replace("__", "|")
        values = tmp.split("|")
        addendum_information["dag_id"] = values[0]
        addendum_information["operator_id"] = values[1]
        addendum_information["start_date"] = values[2]
    if u'ts' in context:
        addendum_information["start_date"] = str(context[u'ts'])
    to_restore = self.push_environment("operator")
    driver_env = None
    executor_env = None
    if  hasattr(self, '_conf'):
        driver_env = self.get_environment("driver")
        if driver_env is not None:
            for key, value in driver_env.items():
                self._conf['spark.yarn.appMasterEnv.'+key] = value
        executor_env = self.get_environment("executor")
        if executor_env is not None:
            for key, value in executor_env.items():
                self._conf['spark.yarn.executorEnv.'+key] = value
                self._conf['spark.executorEnv.'+key] = value
    process_log = self.init_operator(addendum_information, context)
    if driver_env is not None:
        self._conf['spark.executorEnv.PIDCLIENT_PARENTID'] = process_log.meta_information.uid
    if executor_env is not None:
        self._conf['spark.yarn.appMasterEnv.PIDCLIENT_PARENTID'] = process_log.meta_information.uid
        self._conf['spark.yarn.executorEnv.PIDCLIENT_PARENTID'] = process_log.meta_information.uid
    self.on_start_operator(process_log, context)
    self.restore_environment(to_restore)
    if AIRFLOW_ENV_ID in os.environ:
        context['pid_old_env'] = os.environ[AIRFLOW_ENV_ID]
    os.environ[AIRFLOW_ENV_ID] = process_log.meta_information.uid
    context['pid_process_log'] = process_log


def pid_post_execute(self, context, result=None):
    """method called after a successfull execution of a task

    :param context: the context
    :type context: context
    """

    if 'pid_old_env' in context:
        os.environ[AIRFLOW_ENV_ID] = context['pid_old_env']
        del context['pid_old_env']
    else:
        del os.environ[AIRFLOW_ENV_ID]
    self.on_operator_success(context['pid_process_log'], context, result)
    del context['pid_process_log']


def pid_handle_failure(self, context=None):
    """method called in case of failure of a task

    :param context: the context
    :type context: context
    """
    if 'pid_old_env' in context:
        os.environ[AIRFLOW_ENV_ID] = context['pid_old_env']
        del context['pid_old_env']
    else:
        del os.environ[AIRFLOW_ENV_ID]
    self.on_operator_failure(context['pid_process_log'], context,
                             AirflowOperatorIssue("operator issue .. giving up", False))
    del context['pid_process_log']
    if hasattr(self, 'pid_orig_failure_callback'):
        self.pid_orig_failure_callback(self, context)


def pid_handle_retry(self, context=None):
    """method called in case of retrying a task

    :param context: the context
    :type context: context
    """
    if 'pid_old_env' in context:
        os.environ[AIRFLOW_ENV_ID] = context['pid_old_env']
        del context['pid_old_env']
    else:
        del os.environ[AIRFLOW_ENV_ID]
    self.on_operator_failure(context['pid_process_log'], context,
                             AirflowOperatorIssue("operator issue .. retrying later", True))
    if hasattr(self, 'pid_orig_retry_callback'):
        self.pid_orig_retry_callback(self, context)


class DAGWithLogging(DAG):
    """Class that extends a DAG to support logging facility"""

    def init_workflow(self, info, context=None):
        """method that should be overwritten by the user
        to initialize the logging factory ( before the workflow is executed )

        :param process_log: the process log
        :type process_log: ProcessLog
        """
        process_log = LoggingFactory(sysinfo=info).get_logger("-", "AIRFLOW", datetime.now())
        return process_log

    def on_start_workflow(self, process_log, context=None):
        """method that should be overwritten by the user
        to fix information in the process_log before the workflow is
        executed

        :param process_log: the process log
        :type process_log: ProcessLog
        """
        if process_log is not None:
            process_log.proc_started()
        return process_log

    def on_workflow_success(self, process_log, context=None, result=None):
        """method that should be overwritten by the user
        to fix information in the process_log after the workflow has
        run successfully

        :param process_log: the process log
        :type process_log: ProcessLog
        """
        if process_log is not None:
            if result is None:
                process_log.proc_stopped(0, None)
            else:
                process_log.proc_stopped(0, str(result))

    def on_workflow_failure(self, process_log, context=None, received_exception=None):
        """method that should be overwritten by the user
        to fix information in the process_log after the workflow has
        failed

        :param process_log: the process log
        :type process_log: ProcessLog
        """
        if process_log is not None:
            if received_exception is None:
                process_log.proc_stopped(1, "Worflow ends with an issue")
            elif hasattr(received_exception, 'retry') and received_exception.retry:
                process_log.proc_stopped(-1, str(received_exception))
            else:
                process_log.proc_stopped(2, str(received_exception))

    def init_operator(self, info, context=None):
        """method that should be overwritten by the user
        to initiliase the logging factory ( before the operator is
        executed

        :param process_log: the process log
        :type process_log: ProcessLog
        """
        process_log = LoggingFactory(sysinfo=info).get_logger("-", "AIRFLOW", datetime.now())
        return process_log

    def on_start_operator(self, process_log, context=None):
        """method that should be overwritten by the user
        to fix information in the process_log before the operator is
        executed

        :param process_log: the process log
        :type process_log: ProcessLog
        """
        if process_log is not None:
            process_log.proc_started()
        return process_log

    def on_operator_success(self, process_log, context=None, result=None):
        """method that should be overwritten by the user
        to fix information in the process_log after the operator has
        run successfully

        :param process_log: the process log
        :type process_log: ProcessLog
        """
        if process_log is not None:
            if result is None:
                process_log.proc_stopped(0, None)
            else:
                process_log.proc_stopped(0, str(result))

    def on_operator_failure(self, process_log, context=None, received_exception=None):
        """method that should be overwritten by the user
        to fix information in the process_log after the operator has
        failed

        :param process_log: the process log
        :type process_log: ProcessLog
        """
        if process_log is not None:
            if received_exception is None:
                process_log.proc_stopped(1, "Operator ends with an issue")
            elif hasattr(received_exception, 'retry') and received_exception.retry:
                process_log.proc_stopped(-1, str(received_exception))
            else:
                process_log.proc_stopped(2, str(received_exception))

    def get_environment(self, env_type):
        """method to obtain a dictionary of values to set in the environment
        to configure the logging facility

        :param env_type: the environment context
        :type env_type: string ( one of workflow, operator, driver or operator )

        :returns: a dictionary of values to set in the environment
        :rtype: a dictionary
        """
        return None

    def get_operator(self, context):
        """method to get the operator that is currently executed

        :param process_log: the context
        :type process_log: context

        :returns: the operator object
        :rtype: Task

        """
        if context is not None:
            operator = context[u'task_instance'].task
            return operator
        return None

    def log_operator(self, dag):
        """method that patches the present dag to log
        airflow operator information

        :param on_start_action: a function to call before logging the operator with the process_log
        as information to let it adapted by the user
        :type on_start_action: function
        :param on_stop_action: a function to call at the end of the operator with the process_log
        as information to let it adapted by the user
        :type on_stop_action: function
        """

        for task in dag.tasks:
            if hasattr(self, 'on_failure_callback'):
                task.pid_orig_failure_callback = task.on_failure_callback
            task.on_failure_callback = types.MethodType(pid_handle_failure, task)
            if hasattr(self, 'on_retry_callback'):
                task.pid_orig_retry_callback = task.on_retry_callback
            task.on_retry_callback = types.MethodType(pid_handle_retry, task)
            task.pre_execute = types.MethodType(pid_pre_execute, task)
            task.post_execute = types.MethodType(pid_post_execute, task)
            task.init_operator = self.init_operator
            task.get_environment = self.get_environment
            task.push_environment = self.push_environment
            task.restore_environment = self.restore_environment
            task.on_start_operator = self.on_start_operator
            task.on_operator_success = self.on_operator_success
            task.on_operator_failure = self.on_operator_failure

    def push_environment(self, env_type):
        """method that fixes environment variables according the user provided
        get_environment method
        :param env_type: one of ( workflow, operator, driver or operator ) depending
                         on the context from which the method is called.
                         That parameter shall be provided to the user defined method to
                         help the user deciding what to provide.
        :type env_type: string

        :returns: a dictionary of values that needed to be restored
        :rtype: a dictionary
        """
        fixed_env = self.get_environment(env_type)
        to_restore = None
        if fixed_env is not None:
            to_restore = {}
            for key, value in fixed_env.items():
                if key in os.environ:
                    to_restore[key] = os.environ[key]
                else:
                    to_restore[key] = None
                os.environ[key] = value
        return to_restore

    def restore_environment(self, to_restore):
        """method to restore the original environment with the help of the previously saved
        environement
        :param to_restore: the previous values of the environment
        :type to_restore: a dictionary
        """
        if to_restore is not None:
            for key, value in to_restore.items():
                if value is None:
                    del os.environ[key]
                else:
                    os.environ[key] = value

    def run(
            self,
            *args, **kwargs):
        """
            method that overiddes the existing method of a dag class
        """
        addendum_information = {}
        addendum_information["dag_id"] = self.dag_id
        addendum_information["start_date"] = str(kwargs['start_date'])
        addendum_information["end_date"] = str(kwargs['end_date'])

        to_restore = self.push_environment("workflow")
        process_log = self.init_workflow(addendum_information)
        self.on_start_workflow(process_log, None)
        self.restore_environment(to_restore)
        result = None
        try:
            super(DAGWithLogging, self).run(*args, **kwargs)
            self.on_workflow_success(process_log, None, result)
        except (Exception) as error:
            self.on_workflow_failure(process_log, None, error)
            raise error
