"""
Created on 26 mars 2018

@author: Tonneau Stephane
"""

import sys
import os
import datetime
import uuid
import json
import socket
import getpass
import logging
try:
    from shapely import wkt
    from shapely import geometry
except ImportError:
    print("ERROR : Missing required shapely module")


# Utilities
def get_host():
    """Get the address of the host

    :returns: the host name of the computer
    :rtype: string
    """
    host_addr = socket.gethostname()
    if (host_addr is not None) and ("." in host_addr):
        return host_addr
    the_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        # doesn't even have to be reachable
        the_socket.connect(('10.255.255.255', 1))
        host_addr = the_socket.getsockname()[0]
    except Exception:
        host_addr = '127.0.0.1'
    finally:
        the_socket.close()
    return host_addr


def get_user():
    """Get the logged username

    :returns: the username of the logged user
    :rtype: string
    """
    if "USER" in os.environ:
        username = os.environ["USER"]
    elif "LOGNAME" in os.environ:
        username = os.environ["LOGNAME"]
    else:
        try:
            username = getpass.getuser()
        except KeyError:
            username = 'UNKNOWN'
    return username


def read_config(filename):
    """Read a configuration file with the adequate configuration parser

    :param filename: the file name of a configuration file
    :type filename: string
    :returns: a python configuration object with the values present in the file
    :rtype: a python configuration
    """
    configuration = None
    if os.path.isfile(filename):
        if sys.version_info < (3,):
            import ConfigParser
            configuration = ConfigParser.ConfigParser()
        else:
            import configparser
            configuration = configparser.ConfigParser()
        configuration.read(filename)
    else:
        print("ERROR : file " + str(filename) + " is no found")
    return configuration


def get_config(configuration, node_name, name):
    """Return a specific definition element from
    the Configuration or the environment

    :param configuration: the python configuration object
    :type configuration: a python configuration
    :param mode_name: the considered logging system
    :type mode_name: string
    :param name: the name of the parameter from which the value is needed
    :type name: string

    :returns: a python configuration object with the values present in the file
    :rtype: string

    """
    if name is None:
        return None
    if (node_name is None) or (node_name == "main"):
        tmp_env = "PIDCLIENT_" + name.upper()
        mode_name = "main"
    else:
        tmp_env = "PIDCLIENT_" + node_name.upper() + "_" + name.upper()
        mode_name = node_name
    if tmp_env in os.environ:
        value = os.environ[tmp_env]
    elif configuration is not None:
        try:
            value = configuration.get(mode_name, name)
        except:
            value = None
    else:
        value = None
    return value


def first_not_null(*args):
    """Return the first argument that is not None

    :param args: the list of parameters from which the first not None element must be gotten
    :type args: a list
    :returns: the first not None element of the list
    :rtype: string
    """
    for value in args:
        if value is not None:
            return str(value)
    return None


def bean_2_dict(value):
    """Serialize the content of value as a string dictionary

    :param value: an object from which the json representation must be constructed.
    :type value: JsonBased
    :returns: a dictionary representation of the provided object
    :rtype: dictionary
    """
    if value is None:
        result = None
    elif isinstance(value, JsonBased):
        result = value.to_dict()
        if not result:
            result = None
    elif isinstance(value, list):
        result = []
        for subval in value:
            result.append(bean_2_dict(subval))
        if not result:
            result = None
    elif isinstance(value, datetime.datetime):
        result = value.strftime("%Y-%m-%dT%H:%M:%SZ")
    else:
        result = value
    return result


class JsonBased(object):
    """Abstract class that contains json or dictionnary serialization methods"""

    def to_json(self):
        """Serialize the content of the class as a json string

        :returns: the json representation of the object
        :rtype: string
        """
        return json.dumps(self.to_dict())

    def to_dict(self):
        """Serialize the content of the class as a dictionary value,
        withdrawing leading "_"  characters from key name

        :returns: the dictionary representation of the object
        :rtype: dictionary
        """
        result = {}
        for key in self.__dict__:
            tmp_name = key
            if key[0:1] == "_":
                tmp_name = key[1:]
            value = self.__dict__[key]
            tmp_value = bean_2_dict(value)
            if tmp_value is not None:
                result[tmp_name] = tmp_value
        return result


class GeoJSON(JsonBased):
    """class that contains a geojson polygon definition"""

    def __init__(self, wktDefinition):
        """Initialisator of the class

        :param wktDefinition: a wkt representation of a geometry
        :type wktDefinition: string
        """
        self.coordinates = wkt.loads(wktDefinition)

    def to_dict(self):
        """Serialize the content of the class as a dictionary value,
        ignoring keys starting by _

        :returns: the dictionary representation of the object
        :rtype: dictionary
        """
        return geometry.mapping(self.coordinates)

    def to_json(self):
        """Serialize the content of the class as a json string

        :returns: the json representation of the object
        :rtype: string
        """
        return json.dumps(geometry.mapping(self.coordinates))


class SystemEnvironment(JsonBased):
    """Abstract class that encapsulate environment specificities"""

    _my_counter = 1
    _my_node = None

    def __init__(self, **configuration):
        """Initialisator of the class

        :param configuration: specific parameters of the considered environment
        :type configuration: a dictionary
        """
        self.host = get_host()
        self.user = get_user()
        self.configuration = None
        self.job_id = None
        self.job_type = None
        self.main_id = None
        if "ENVIRONMENT" in os.environ:
            self.environment_type = os.environ["ENVIRONMENT"]
        else:
            self.environment_type = None
        if ("PIDCLIENT_PARENTID" in os.environ) and (len(os.environ["PIDCLIENT_PARENTID"]) > 4):
            self.parent_id = os.environ["PIDCLIENT_PARENTID"]
        else:
            self.parent_id = None

    def get_uid(self):
        """Build a unique id

        :returns: a unique id for the object
        :rtype: string
        """
        uid = str(uuid.uuid1())
        uid = uid + "-" + str(os.getpid())+str(SystemEnvironment._my_counter)
        SystemEnvironment._my_counter += 1
        # Build a random unique id
        return uid

    def get_node(self):
        """get the node reference

        :returns: the node reference of the computer
        :rtype: string
        """
        if SystemEnvironment._my_node is None:
            SystemEnvironment._my_node = str(uuid.getnode())
        return SystemEnvironment._my_node


class Configuration(JsonBased):
    """Abstract class that contains relevant System configuration information"""

    def __init__(self, driver_memory=None, executor_memory=None, num_executors=None,
                 queue=None, executor_environment=None):
        """Initialisator of the class

        :param driver_memory: memory used in the context of a spark driver
        :type driver_memory: string
        :param executor_memory: memory used in the context of a spark executor
        :type executor_memory: string
        :param num_executors: number of executors value
        :type num_executors: string
        :param init_executors: initial number of executors value
        :type init_executors: string
        :param min_executors: minimum number of executors value
        :type min_executors: string
        :param max_executors: maximum number of executors value
        :type max_executors: string
        :param executor_environment: Executor environment
        :type executor_environment: string
        :param queue: queue value
        :type queue: string
        """
        self.driver_memory = driver_memory
        self.executor_memory = executor_memory
        self.num_executors = num_executors
        self.init_executors = None
        self.min_executors = None
        self.max_executors = None
        self.executor_environment = executor_environment
        self.queue = queue


class LoggingMethod(object):
    """Abstract class that logs information with a specific approach"""

    CONFIG_KEY = "configuration"

    py_log = logging.getLogger(__name__)

    def __init__(self, **parameters):
        """Initialisator of the class

        :param parameters: the general logging parameters
        :type parameters: a python dictionary
        """
        self.logging_type = None
        self.configuration = None
        self.parameters = parameters
        if self.CONFIG_KEY in parameters:
            self.configuration = parameters[self.CONFIG_KEY]

    def get_config(self, node_name, name):
        """Return a specific definition element from
        the Configuration or the environment

        :param mode_name: the considered logging system
        :type mode_name: string
        :param name: the name of the parameter from which the value is needed
        :type name: string

        :returns: a python configuration object with the values present in the file
        :rtype: string

        """
        if name in self.parameters:
            return_val = self.parameters[name]
        else:
            return_val = get_config(self.configuration, node_name, name)

        self.py_log.debug("get_config({}, {}) = {}".format(node_name, name, str(return_val)))

        return return_val
