"""
Created on 26 mars 2018

@author: Tonneau Stephane
"""
import os
from pidclient.base_utils import first_not_null
from pidclient.base_utils import Configuration
from pidclient.base_utils import SystemEnvironment


DEFAULTS = {"spark.driver.memory": None, "spark.executor.memory": None,
            "spark.yarn.queue": None, "spark.dynamicAllocation.minExecutors": None,
            "spark.dynamicAllocation.maxExecutors": None,
            "spark.executor.instances": None}

SPARK_DRIVER_PTYPE = "spark_driver"
SPARK_EXECUTOR_PTYPE = "spark_executor"


def get_conf_value(conf, key):
    """
    Get a specific SparkConf value.  This function return None when
    conf object is None.

    :param conf: configuration of the spark environment
    :type conf: SparkConf
    :param key: the whished key value
    :type key: string
    """
    if conf is None:
        return None
    return conf.get(key)


class YarnClusterConfiguration(Configuration):
    """Class that encapsulates a Spark Configuration in cluster mode."""

    def complete(self, conf):
        """
        Complete the configuration information with
        the environments definitions (or default when not available)
        for spark configuration running in cluster mode.

        :param conf: spark configuration within the cluster spark environment
        :type conf: pyspark.SparkConf
        """

        self.driver_memory = first_not_null(self.driver_memory, get_conf_value(conf, "spark.driver.memory"),
                                            DEFAULTS['spark.driver.memory'])
        self.executor_memory = first_not_null(
            self.executor_memory,
            get_conf_value(conf, "spark.executor.memory"),
            DEFAULTS['spark.executor.memory'])
        # if conf.get("spark.master") is not None:
        #    self.spark_master = first_not_null(self.spark_master, str(conf.get("spark.master")))
        # if conf.get("spark.submit.deployMode") is not None:
        #    self.deploy_mode = str(conf.get("spark.submit.deployMode"))
        if get_conf_value(conf, "spark.dynamicAllocation.enabled") == "true":
            self.min_executors = first_not_null(get_conf_value(conf, "spark.dynamicAllocation.minExecutors"),
                                                DEFAULTS["spark.dynamicAllocation.minExecutors"])
            self.max_executors = first_not_null(get_conf_value(conf, "spark.dynamicAllocation.maxExecutors"),
                                                DEFAULTS["spark.dynamicAllocation.maxExecutors"])
            self.num_executors = first_not_null(
                self.num_executors, self.min_executors)
        else:
            self.num_executors = first_not_null(self.num_executors,
                                                get_conf_value(conf, "spark.executor.instances"),
                                                DEFAULTS["spark.executor.instances"])
            self.min_executors = None
            self.max_executors = None

        self.num_executors = self.min_executors
        self.queue = first_not_null(self.queue,
                                    get_conf_value(conf, "spark.yarn.queue"),
                                    DEFAULTS["spark.yarn.queue"])
        if get_conf_value(conf, "spark.yarn.isPython") == "true":
            if get_conf_value(conf, "spark.executorEnv.PYTHONPATH") is not None:
                self.executor_environment = first_not_null(self.executor_environment, str(
                    get_conf_value(conf, "spark.executorEnv.PYTHONPATH")))
        return self


class YarnEnvironment(SystemEnvironment):
    """Class that captures spark environment particularities"""

    def __init__(self, **configuration):
        """Initialisator of the class

        :param configuration: specific parameters of the airflow environment
        :type configuration: a dictionary
        """

        super(YarnEnvironment, self).__init__(**configuration)
        if ("APP_SUBMIT_TIME_ENV" in os.environ) or ("APPLICATION_WEB_PROXY_BASE_ENV" in os.environ):
            self.job_type = SPARK_DRIVER_PTYPE
            self.configuration = YarnClusterConfiguration()
            conf = None
            try:
                from pyspark import SparkConf
                conf = SparkConf(True)
            except Exception:
                # We can not get the required configuration
                # We will ignore this.
                pass
            self.configuration.complete(conf)
        else:
            self.job_type = SPARK_EXECUTOR_PTYPE

        if "SPARK_USER" in os.environ:
            self.user = os.environ["SPARK_USER"]

        # Extract required spark ID information into a dictionary
        if "HADOOP_TOKEN_FILE_LOCATION" in os.environ:
            splited_info = os.environ["HADOOP_TOKEN_FILE_LOCATION"].split("/")
            i = len(splited_info) - 1
            while i >= 0:
                if splited_info[i].startswith("application_"):
                    if self.main_id is None:
                        self.main_id = splited_info[i][12:]
                    self.job_id = splited_info[i]
                elif splited_info[i].startswith("container_"):
                    if self.job_id is None:
                        self.job_id = splited_info[i]
                    self.main_id = splited_info[i][10:]
                i = i - 1
        elif (configuration is not None) and ('application_id' in configuration):
            self.job_id = configuration['application_id']
            self.main_id = self.job_id[12:]
        return
