"""
Created on 10 avril 2018

@author: Tonneau Stephane
"""
import pidclient.pid_entry as pid_entry


class ProcessLog():
    """Class that allow the logging of the relevant process log information"""

    def __init__(self, pid, meta_information, logger):
        """
        Initialisator of the class

        :param pid: the standard information to log
        :type pid: PidEntry
        :param meta_information: meta information about the information to log
        :type meta_information: Metainformation
        :param logger: a list of Logging methods to use for logging
        :type logger: list<LoggingMethod>
        """
        self.pid_entry = pid
        """the standard information to log"""
        self.meta_information = meta_information
        """meta_information: meta information about the information to log"""
        self.logging_methods = logger

    def __close_logging(self):
        """close the different logging systems

        :returns: the object itself
        :rtype: ProcessLog
        """
        for key in self.logging_methods:
            key.close()
        return self

    def __write_log(self):
        """write the logs in the different logging systems

        :returns: the object itself
        :rtype: ProcessLog
        """
        for key in self.logging_methods:
            key.process_log(self.pid_entry, self.meta_information)
        return self

    def add_file(self, path, context,
                 type_value=None, geometry=None,
                 checksum=None, size=None, product_id=None, ogc_urn=None):
        """add a file information to the log entry

        :param path: absolute path to a single file or directory
        :type path: string
        :param context: the context associated with the file
        :type context: string
        :param type_value: the media type associated with the file
        :type type_value: string
        :param geometry: the geometry in the GeoJSON format
        :type geometry: wkt geometry
        :param checksum: the checksum of the file in the format {[SHA|MD5|..]}<value>
        :type checksum: string
        :param size: the total size of the file or directory
        :type size: int
        :param product_id: the product_id of the product to which the file or directory refers
        :type product_id: string
        :param ogc_urn: the ogc urn reference of the product to which the file or directory refers
        :type ogc_urn: string

        :returns: the object itself
        :rtype: ProcessLog
        """
        file_info = pid_entry.FileInfo(path, context, type_value, geometry,
                                       checksum, size, product_id, ogc_urn)
        self.pid_entry.files.append(file_info)
        return self

    def add_files(self, paths, context,
                  type_value=None, geometry=None,
                  product_id=None, ogc_urn=None):
        """add a file information to the log entry

        :param path: array of absolute path to files or directories
        :type path: string[]
        :param context: the context associated with the files/directories
        :type context: string
        :param type_value: the media type associated with the file
        :type type_value: string
        :param geometry: the geometry in the GeoJSON format
        :type geometry: wkt geometry
        :param product_id: the product_id of the product to which the file or directory refers
        :type product_id: string
        :param ogc_urn: the ogc urn reference of the product to which the file or directory refers
        :type ogc_urn: string

        :returns: the object itself
        :rtype: ProcessLog
        """

        if paths is not None:
            for path in paths:
                self.add_file(path, context, type_value, geometry, None, None, product_id, ogc_urn)
        return self

    def add_trace(self, trace):
        """add a trace information to the log entry

        :param trace: an execution trace (commands with parameters, ...)
        :type trace: string

        """
        trace_info = pid_entry.TraceInfo(trace)
        self.pid_entry.execution_info.append(trace_info)
        return self

    def fix_trace(self):
        """add traces information based of the process
        hierarchy to the log entry

        """
        try:
            import psutil
            process = psutil.Process()
            nbr = 10
            while (process is not None) and  (nbr > 0):
                self.add_trace(process.cmdline())
                nbr = nbr - 1
                process = process.parent()
        except:
            pass


    def proc_started(self):
        """write the logs in the different logging systems

        :returns: the object itself
        :rtype: ProcessLog
        """
        self.fix_spark_context()
        self.__write_log()
        return self

    def fix_spark_context(self, spark_context=None):
        """update the different information according sparkcontext
        in the context of the yarn client mode

        :param spark_context: the sparkContext
        :type spark_context: sparkContext

        :returns: the object itself
        :rtype: ProcessLog
        """

        if self.pid_entry.job_type == 'spark_cdriver':
            try:
                if spark_context is None:
                    from pyspark import SparkContext
                    if SparkContext._active_spark_context is not None:
                        self.pid_entry._job_id = str(SparkContext._active_spark_context.applicationId)
                        self.pid_entry.job_type = 'spark_driver'
                else:
                    self.pid_entry._job_id = str(spark_context.applicationId)
                    self.pid_entry.job_type = 'spark_driver'
            except Exception:
                # We can not get the required context
                # We will ignore this.
                pass
        return self

    def proc_stopped(self, exit_code, message=None):
        """update the end time and write the logs in the different logging systems

        :param exit_code: the exit code number of the process
        :type exit_code: int
        :param message: a string that gives more information on the end of the process
        :type message: string

        :returns: the object itself
        :rtype: ProcessLog
        """
        if message is not None:
            self.pid_entry.message = message
        if exit_code is not None:
            self.pid_entry.exit_code = str(exit_code)
        self.fix_spark_context()
        self.pid_entry.finalise()
        self.__write_log()
        self.__close_logging()
        return self
